/*
Copyright 2019, Thierry Masson, Bruno Iochum

Contact: thierry.masson@cpt.univ-mrs.fr

This file is part of a computer program whose purpose is to help to compute
asymptotic expansion coefficients of the heat-trace for a nonminimal Laplace type operator
using a method developed and exposed in the paper
"Heat coefficient a_4 for nonminimal Laplace type operators" by Bruno Iochum and Thierry Masson.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

let REL_PATH = '../';

let startTime = Date.now();

const fs = require('fs');
const path = require('path');
const chalk = require('chalk');
const {GLOBAL, CONST, DISPLAY_SYMBOLS, OPT, FILES} = require(REL_PATH + 'INIT-FILES/options');
const {FH} = require(REL_PATH + 'FUNCTIONS-FILES/functions-helpers');

const {
				Polynomials,
				RationalExpression
			} = require(REL_PATH + 'MATH-OBJECTS/MathRationalExpression');

const {
				MathElement,
				MathOperator,
				MathElement_one,
			} = require(REL_PATH + 'MATH-OBJECTS/MathElement');

const {
				MathExpression,
				MathProduct,
				MathTensorProduct,
				MathSum,
				MathArgument
			} = require(REL_PATH + 'MATH-OBJECTS/MathExpression');

const {PROTO, SUBST, RULE_DATA, SELECTION} = require(REL_PATH + 'INIT-FILES/initialization-data');

const {FC} = require(REL_PATH + 'FUNCTIONS-FILES/functions-computation');
const {FR} = require(REL_PATH + 'FUNCTIONS-FILES/functions-rules');
const {FS} = require(REL_PATH + 'FUNCTIONS-FILES/functions-substitutions');
const {FS_DATA} = require(REL_PATH + 'INIT-FILES/options-substitutions');


FH.showTitle('BEGINNING: ' + path.basename(__filename));


FH.showSubtitle('COMPUTATION OF SUBSTITUTIONS OF H');


// from these data, the code should generate all the substitutions in normal coordinates
// data given at level 0 of derivation…
let initialSubstitutionH = {
		optLaTeX: OPT.LTX,
		termElement: CONST.MAIN_ELEMENT,
		filePath: FILES.SUBST.H_to_nc, // file in which substitution data are saved
		sumFileRoot: './RESULTS/SUBSTITUTIONS/SubstitutionsH-sum', // file root to save sums
		latexLetter: 'H',
		orgItem: PROTO.H,
		derivationOperator: PROTO.nabla,
		minLevel: 0, // in final substitution data
		maxLevel: 4, // in final substitution data
		newItems: // list all new items
			[
				{
					item: PROTO.H_0, // MathProduct
					// all mapIndices should have same "from" in same order
					mapIndices: [
						{from:[0], to:[0,0]},
						{from:[1], to:[0,1]}
					],
				}
			]
	};


let latexToSaveH = FS.createSubstitutionsDataNormCoord(initialSubstitutionH, FS_DATA.operatorData, FS_DATA.placeHolderTensorData, REL_PATH);


// save latex of computations done in compute-substitutions-read-from-file.js
// FH.saveToFileWithSureFolder(REL_PATH + 'LATEX/SUBSTITUTIONS/substitutions-computed-H.tex', latexToSaveH);




FH.showTitle('END: ' + path.basename(__filename));
