/*
Copyright 2019, Thierry Masson, Bruno Iochum

Contact: thierry.masson@cpt.univ-mrs.fr

This file is part of a computer program whose purpose is to help to compute
asymptotic expansion coefficients of the heat-trace for a nonminimal Laplace type operator
using a method developed and exposed in the paper
"Heat coefficient a_4 for nonminimal Laplace type operators" by Bruno Iochum and Thierry Masson.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

let REL_PATH = '../';

let startTime = Date.now();

const fs = require('fs');
const path = require('path');
const chalk = require('chalk');
const {GLOBAL, CONST, DISPLAY_SYMBOLS, OPT, FILES} = require(REL_PATH + 'INIT-FILES/options');
const {FH} = require(REL_PATH + 'FUNCTIONS-FILES/functions-helpers');

const {
				Polynomials,
				RationalExpression
			} = require(REL_PATH + 'MATH-OBJECTS/MathRationalExpression');

const {
				MathElement,
				MathOperator,
				MathElement_one,
			} = require(REL_PATH + 'MATH-OBJECTS/MathElement');

const {
				MathExpression,
				MathProduct,
				MathTensorProduct,
				MathSum,
				MathArgument
			} = require(REL_PATH + 'MATH-OBJECTS/MathExpression');

const {PROTO, SUBST, RULE_DATA, SELECTION} = require(REL_PATH + 'INIT-FILES/initialization-data');

const {FC} = require(REL_PATH + 'FUNCTIONS-FILES/functions-computation');
const {FR} = require(REL_PATH + 'FUNCTIONS-FILES/functions-rules');
const {FS} = require(REL_PATH + 'FUNCTIONS-FILES/functions-substitutions');
const {FS_DATA} = require(REL_PATH + 'INIT-FILES/options-substitutions');


FH.showTitle('BEGINNING: ' + path.basename(__filename));

// create tensor on which to test the propagation of Riemann derivatives
let a = new MathElement();
a.setID('a')
	.setLetter('a')
	.setLaTeXLetter('a')
	.setProductKindToNonCommutative()
	.addIndiceUp()
	.addIndiceUp()
	.addIndiceDown()
	.setSumWeight(100)
	.setProductWeight(100);


FH.showSubtitle('COMPUTATION OF SUBSTITUTIONS OF HNABLA ON N');

// initialize data to use

// FS_DATA.operatorData.Christoffel.showConsole(OPT.SHOW, '𝚪 = ');

// console.log('');

let initialDataNGamma = {
		optLaTeX: OPT.LTX,
		termElement: CONST.MAIN_ELEMENT,
		filePath: FILES.SUBST.hnabla_N_to_Gamma, // file in which substitution data are saved
		sumFileRoot: './RESULTS/SUBSTITUTIONS/Substitutions-hnabla-N-Gamma-sum', // file root to save sums
		toNormalCoordinates: false,
		fileSubstitutionsGammaToNC: FILES.SUBST.Gamma_to_nc,
		latexLetter: 'N',
		orgItem: PROTO.N,
		// orgItem: a,
		derivationOperator: PROTO.nabla,
		minLevel: 1, // in final substitution data
		maxLevel: 3, // in final substitution data
	};

let initialDataNNC = {
		optLaTeX: OPT.LTX,
		termElement: CONST.MAIN_ELEMENT,
		filePath: FILES.SUBST.hnabla_N_to_nc, // file in which substitution data are saved
		sumFileRoot: './RESULTS/SUBSTITUTIONS/Substitutions-hnabla-N-NC-sum', // file root to save sums
		toNormalCoordinates: true,
		fileSubstitutionsGammaToNC: FILES.SUBST.Gamma_to_nc,
		latexLetter: 'N',
		orgItem: PROTO.N,
		// orgItem: a,
		derivationOperator: PROTO.nabla,
		minLevel: 1, // in final substitution data
		maxLevel: 3, // in final substitution data
	};

// here the applied operator is 'hnabla'
FS_DATA.operatorData.AppliedOperator = PROTO.hnabla;

let latexToSave = '';

// TO GAMMA
// use the function
latexToSave = FS.createSubstitutionsDataGaugeToRiemann(initialDataNGamma, FS_DATA.operatorData, FS_DATA.placeHolderTensorData, REL_PATH)

// save latex of computations
FH.saveToFileWithSureFolder(REL_PATH + 'LATEX/SUBSTITUTIONS/substitutions-computed-hnabla-N-Gamma.tex', latexToSave);

// TO NC
// use the function
latexToSave = FS.createSubstitutionsDataGaugeToRiemann(initialDataNNC, FS_DATA.operatorData, FS_DATA.placeHolderTensorData, REL_PATH)

// save latex of computations
FH.saveToFileWithSureFolder(REL_PATH + 'LATEX/SUBSTITUTIONS/substitutions-computed-hnabla-N-NC.tex', latexToSave);



FH.showTitle('END: ' + path.basename(__filename));
