/*
Copyright 2019, Thierry Masson, Bruno Iochum

Contact: thierry.masson@cpt.univ-mrs.fr

This file is part of a computer program whose purpose is to help to compute
asymptotic expansion coefficients of the heat-trace for a nonminimal Laplace type operator
using a method developed and exposed in the paper
"Heat coefficient a_4 for nonminimal Laplace type operators" by Bruno Iochum and Thierry Masson.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

let REL_PATH = '../';

let startTime = Date.now();

const fs = require('fs');
const path = require('path');
const chalk = require('chalk');
const {GLOBAL, CONST, DISPLAY_SYMBOLS, OPT, FILES} = require(REL_PATH + 'INIT-FILES/options');
const {FH} = require(REL_PATH + 'FUNCTIONS-FILES/functions-helpers');

const {
				Polynomials,
				RationalExpression
			} = require(REL_PATH + 'MATH-OBJECTS/MathRationalExpression');

const {
				MathElement,
				MathOperator,
				MathElement_one,
			} = require(REL_PATH + 'MATH-OBJECTS/MathElement');

const {
				MathExpression,
				MathProduct,
				MathTensorProduct,
				MathSum,
				MathArgument
			} = require(REL_PATH + 'MATH-OBJECTS/MathExpression');

const {PROTO, SUBST, RULE_DATA, SELECTION} = require(REL_PATH + 'INIT-FILES/initialization-data');

const {FC} = require(REL_PATH + 'FUNCTIONS-FILES/functions-computation');

let selection = null;
let selectionSum = null;
let partialStartTime = null;
let partialEndTime = null;
let durationTime = null;

FH.showTitle('BEGINNING: ' + path.basename(__filename));


FH.showSubtitle('INITIALIZATION');

FH.showSubsubtitle('Restore from file');

partialStartTime = Date.now();

let sumResult = FC.restoreFromFile(REL_PATH + FILES.JSON.R4_ns[0]);
sumResult.fillNormalizedIndicesArray();
// sumResult.showNormalized(OPT.SHOW, 'R = ');

console.log('The expression contains ' + sumResult.getListLength() + ' terms');

partialEndTime = Date.now();
durationTime = partialEndTime - partialStartTime;
console.log('Duration for restauration (ms): ' + durationTime);


FH.showSubtitle('COMPUTATIONS: OPERATOR RULES…');

FH.showSubsubtitle('Apply operator rules');

let somethingFound = true;
let workingSum = sumResult;
// let loopC = 0;
while(somethingFound) {
	// loopC++;
	// console.log('loopC: ' + loopC);
	// let {flag, sum}
	// 				= FC.applyOperatorSimplificationTopDown(workingSum, PROTO.u);
	let {flag, sum}
					= FC.applyOperatorSimplificationIgnoreDegree(workingSum, PROTO.u);
	workingSum = sum;
	somethingFound = flag;
};

sumResult = workingSum;

sumResult.updateExpression();
sumResult.sortBySumWeight();
sumResult.fillNormalizedIndicesArray();
sumResult.simplifyExpression();
// sumResult.showNormalized(OPT.SHOW, 'R = ');
// console.log('');
console.log('The expression contains ' + sumResult.getListLength() + ' terms')



FH.showSubsubtitle('Save simplified result');
// save result
FH.saveItemToFile(sumResult, REL_PATH + FILES.JSON.R4[0]);



FH.showSubtitle('COMPUTATIONS: N=0');


FH.showSubsubtitle('Substitutions of N to 0: simplified');

sumResult.substituteItemAndDerivativesWithData(SUBST.N_TO_ZERO);
sumResult.sortBySumWeight();
sumResult.fillNormalizedIndicesArray();
sumResult.simplifyExpression();
// sumResult.showNormalized(OPT.SHOW, 'R(N=0) = ');
// console.log('');
console.log('The expression contains ' + sumResult.getListLength() + ' terms')


FH.showSubsubtitle('Save result');
// save result
FH.saveItemToFile(sumResult, REL_PATH + FILES.JSON.R4_N0[0]);





let endTime = Date.now();
durationTime = endTime - startTime;
console.log('Duration (ms): ' + durationTime);


FH.showTitle('END: ' + path.basename(__filename));
