/*
Copyright 2019, Thierry Masson, Bruno Iochum

Contact: thierry.masson@cpt.univ-mrs.fr

This file is part of a computer program whose purpose is to help to compute
asymptotic expansion coefficients of the heat-trace for a nonminimal Laplace type operator
using a method developed and exposed in the paper
"Heat coefficient a_4 for nonminimal Laplace type operators" by Bruno Iochum and Thierry Masson.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

let REL_PATH = '../';

let startTime = Date.now();

const fs = require('fs');
const path = require('path');
const chalk = require('chalk');
const {GLOBAL, CONST, DISPLAY_SYMBOLS, OPT, FILES} = require(REL_PATH + 'INIT-FILES/options');
const {FH} = require(REL_PATH + 'FUNCTIONS-FILES/functions-helpers');

const {
				Polynomials,
				RationalExpression
			} = require(REL_PATH + 'MATH-OBJECTS/MathRationalExpression');

const {
				MathElement,
				MathOperator,
				MathElement_one,
			} = require(REL_PATH + 'MATH-OBJECTS/MathElement');

const {
				MathExpression,
				MathProduct,
				MathTensorProduct,
				MathSum,
				MathArgument
			} = require(REL_PATH + 'MATH-OBJECTS/MathExpression');

const {PROTO, SUBST, RULE_DATA, SELECTION} = require(REL_PATH + 'INIT-FILES/initialization-data');

const {FC} = require(REL_PATH + 'FUNCTIONS-FILES/functions-computation');
const {FI} = require(REL_PATH + 'FUNCTIONS-FILES/functions-input');

let selection = null;
let selectionSum = null;
let partialStartTime = null;
let partialEndTime = null;
let durationTime = null;


FH.showTitle('BEGINNING: ' + path.basename(__filename));


partialStartTime = Date.now();

FH.showSubtitle('GET SAVED RESULTS FOR R4 2');

FH.showSubsubtitle('Restore simplified expression');

// simplified expression with N non zero
let sum2Result = FC.restoreFromFile(REL_PATH + FILES.JSON.R4[2]);

sum2Result.sortBySumWeight();
sum2Result.fillNormalizedIndicesArray();
sum2Result.simplifyExpression();

console.log('');
console.log('The expression contains ' + sum2Result.getListLength() + ' terms')
console.log('');


FH.showSubsubtitle('Change derivatives');

// substitute cumulative derivations on 'u' with "powered" derivations
sum2Result.substituteItemAndDerivativesWithData(SUBST.HNABLAu);
sum2Result.sortBySumWeight();
sum2Result.fillNormalizedIndicesArray();
sum2Result.simplifyExpression();

// substitute cumulative derivations on 'q' with "powered" derivations
sum2Result.substituteItemAndDerivativesWithData(SUBST.HNABLAq);
sum2Result.sortBySumWeight();
sum2Result.fillNormalizedIndicesArray();
sum2Result.simplifyExpression();

// substitute cumulative derivations on 'N' with "powered" derivations
sum2Result.substituteItemAndDerivativesWithData(SUBST.HNABLAN);
sum2Result.sortBySumWeight();
sum2Result.fillNormalizedIndicesArray();
sum2Result.simplifyExpression();


FH.showSubsubtitle('Split according to patterns');


FC.saveSplitSum(sum2Result, PROTO.u, '-2', REL_PATH);








FH.showSubtitle('GET SAVED RESULTS FOR R4 3');

FH.showSubsubtitle('Restore simplified expression');

// simplified expression with N non zero
let sum3Result = FC.restoreFromFile(REL_PATH + FILES.JSON.R4[3]);

sum3Result.sortBySumWeight();
sum3Result.fillNormalizedIndicesArray();
sum3Result.simplifyExpression();

console.log('');
console.log('The expression contains ' + sum3Result.getListLength() + ' terms')
console.log('');


FH.showSubsubtitle('Change derivatives');

// substitute cumulative derivations on 'u' with "powered" derivations
sum3Result.substituteItemAndDerivativesWithData(SUBST.HNABLAu);
sum3Result.sortBySumWeight();
sum3Result.fillNormalizedIndicesArray();
sum3Result.simplifyExpression();

// substitute cumulative derivations on 'q' with "powered" derivations
sum3Result.substituteItemAndDerivativesWithData(SUBST.HNABLAq);
sum3Result.sortBySumWeight();
sum3Result.fillNormalizedIndicesArray();
sum3Result.simplifyExpression();

// substitute cumulative derivations on 'N' with "powered" derivations
sum3Result.substituteItemAndDerivativesWithData(SUBST.HNABLAN);
sum3Result.sortBySumWeight();
sum3Result.fillNormalizedIndicesArray();
sum3Result.simplifyExpression();


FH.showSubsubtitle('Split according to patterns');


FC.saveSplitSum(sum3Result, PROTO.u, '-3', REL_PATH);







FH.showSubtitle('GET SAVED RESULTS FOR R4 4');

FH.showSubsubtitle('Restore simplified expression');

// simplified expression with N non zero
let sum4Result = FC.restoreFromFile(REL_PATH + FILES.JSON.R4[4]);

sum4Result.sortBySumWeight();
sum4Result.fillNormalizedIndicesArray();
sum4Result.simplifyExpression();

console.log('');
console.log('The expression contains ' + sum4Result.getListLength() + ' terms')
console.log('');


FH.showSubsubtitle('Change derivatives');

// substitute cumulative derivations on 'u' with "powered" derivations
sum4Result.substituteItemAndDerivativesWithData(SUBST.HNABLAu);
sum4Result.sortBySumWeight();
sum4Result.fillNormalizedIndicesArray();
sum4Result.simplifyExpression();

// substitute cumulative derivations on 'q' with "powered" derivations
sum4Result.substituteItemAndDerivativesWithData(SUBST.HNABLAq);
sum4Result.sortBySumWeight();
sum4Result.fillNormalizedIndicesArray();
sum4Result.simplifyExpression();

// substitute cumulative derivations on 'N' with "powered" derivations
sum4Result.substituteItemAndDerivativesWithData(SUBST.HNABLAN);
sum4Result.sortBySumWeight();
sum4Result.fillNormalizedIndicesArray();
sum4Result.simplifyExpression();


FH.showSubsubtitle('Split according to patterns');


FC.saveSplitSum(sum4Result, PROTO.u, '-4', REL_PATH);


let endTime = Date.now();
durationTime = endTime - startTime;
console.log('Duration (ms): ' + durationTime);


FH.showTitle('END: ' + path.basename(__filename));
