/*
Copyright 2019, Thierry Masson, Bruno Iochum

Contact: thierry.masson@cpt.univ-mrs.fr

This file is part of a computer program whose purpose is to help to compute
asymptotic expansion coefficients of the heat-trace for a nonminimal Laplace type operator
using a method developed and exposed in the paper
"Heat coefficient a_4 for nonminimal Laplace type operators" by Bruno Iochum and Thierry Masson.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

let REL_PATH = '../';

let startTime = Date.now();

const fs = require('fs');
const path = require('path');
const chalk = require('chalk');
const {GLOBAL, CONST, DISPLAY_SYMBOLS, OPT, FILES} = require(REL_PATH + 'INIT-FILES/options');
const {FH} = require(REL_PATH + 'FUNCTIONS-FILES/functions-helpers');

const {
				Polynomials,
				RationalExpression
			} = require(REL_PATH + 'MATH-OBJECTS/MathRationalExpression');

const {
				MathElement,
				MathOperator,
				MathElement_one,
			} = require(REL_PATH + 'MATH-OBJECTS/MathElement');

const {
				MathExpression,
				MathProduct,
				MathTensorProduct,
				MathSum,
				MathArgument
			} = require(REL_PATH + 'MATH-OBJECTS/MathExpression');

const {PROTO, SUBST, RULE_DATA, SELECTION} = require(REL_PATH + 'INIT-FILES/initialization-data');

const {FC} = require(REL_PATH + 'FUNCTIONS-FILES/functions-computation');

let selection = null;
let selectionSum = null;
let partialStartTime = null;
let partialEndTime = null;
let durationTime = null;

FH.showTitle('BEGINNING: ' + path.basename(__filename));

let sumResult = null;
let latex = '';

partialStartTime = Date.now();


FH.showSubtitle('N non zero');


let collectedSum = new MathSum();

FH.showSubsubtitle('Nabla 0');

// nabla0, use the simplified expression
sumResult = FC.restoreFromFile(REL_PATH + FILES.JSON.R4[0]);
sumResult.fillNormalizedIndicesArray();

console.log('The expression contains ' + sumResult.getListLength() + ' terms')

// u => parallel
sumResult.substituteItemAndDerivativesWithData(SUBST.uToConstant);
sumResult.sortBySumWeight();
sumResult.fillNormalizedIndicesArray();
sumResult.simplifyExpression();

// substitute cumulative derivations on 'q' with "powered" derivations
sumResult.substituteItemAndDerivativesWithData(SUBST.HNABLAq);
sumResult.sortBySumWeight();
sumResult.fillNormalizedIndicesArray();
sumResult.simplifyExpression();

// substitute cumulative derivations on 'N' with "powered" derivations
sumResult.substituteItemAndDerivativesWithData(SUBST.HNABLAN);
sumResult.sortBySumWeight();
sumResult.fillNormalizedIndicesArray();
sumResult.simplifyExpression();


console.log('');
sumResult.showNormalized(OPT.SHOW, 'R(u=parallel) = ');

console.log('');
console.log('The expression contains ' + sumResult.getListLength() + ' terms')

latex = sumResult.getLaTeXEquation(OPT.LTX, 'R_{4,0}(u=\\text{parallel}) = ');
// console.log(latex);
FH.saveToFileWithSureFolder(REL_PATH + FILES.TEX.R4_0_u_parallel, latex);


collectedSum.addTo(sumResult);




// Nabla 1 is already 0

FH.showSubsubtitle('Nabla 2');


// nabla2, use the simplified expression
sumResult = FC.restoreFromFile(REL_PATH + FILES.JSON.R4[2]);
sumResult.fillNormalizedIndicesArray();

console.log('The expression contains ' + sumResult.getListLength() + ' terms')

// u => parallel
sumResult.substituteItemAndDerivativesWithData(SUBST.uToConstant);
sumResult.sortBySumWeight();
sumResult.fillNormalizedIndicesArray();
sumResult.simplifyExpression();

// substitute cumulative derivations on 'N' with "powered" derivations
sumResult.substituteItemAndDerivativesWithData(SUBST.HNABLAN);
sumResult.sortBySumWeight();
sumResult.fillNormalizedIndicesArray();
sumResult.simplifyExpression();

console.log('');
sumResult.showNormalized(OPT.SHOW, 'R(u=parallel) = ');

console.log('');
console.log('The expression contains ' + sumResult.getListLength() + ' terms')

latex = sumResult.getLaTeXEquation(OPT.LTX, 'R_{4,2}(u=\\text{parallel}) = ');
// console.log(latex);
FH.saveToFileWithSureFolder(REL_PATH + FILES.TEX.R4_2_u_parallel, latex);


collectedSum.addTo(sumResult);



FH.showSubsubtitle('Nabla 3');

// nabla3, use the simplified expression
sumResult = FC.restoreFromFile(REL_PATH + FILES.JSON.R4[3]);
sumResult.fillNormalizedIndicesArray();

console.log('The expression contains ' + sumResult.getListLength() + ' terms')

// u => parallel
sumResult.substituteItemAndDerivativesWithData(SUBST.uToConstant);
sumResult.sortBySumWeight();
sumResult.fillNormalizedIndicesArray();
sumResult.simplifyExpression();

// substitute cumulative derivations on 'N' with "powered" derivations
sumResult.substituteItemAndDerivativesWithData(SUBST.HNABLAN);
sumResult.sortBySumWeight();
sumResult.fillNormalizedIndicesArray();
sumResult.simplifyExpression();

console.log('');
sumResult.showNormalized(OPT.SHOW, 'R(u=parallel) = ');

console.log('');
console.log('The expression contains ' + sumResult.getListLength() + ' terms')

latex = sumResult.getLaTeXEquation(OPT.LTX, 'R_{4,3}(u=\\text{parallel}) = ');
// console.log(latex);
FH.saveToFileWithSureFolder(REL_PATH + FILES.TEX.R4_3_u_parallel, latex);


collectedSum.addTo(sumResult);


FH.showSubsubtitle('Nabla 4');

// nabla4, use the simplified expression with N non zero
sumResult = FC.restoreFromFile(REL_PATH + FILES.JSON.R4[4]);
sumResult.fillNormalizedIndicesArray();

console.log('The expression contains ' + sumResult.getListLength() + ' terms')

// u => parallel
sumResult.substituteItemAndDerivativesWithData(SUBST.uToConstant);
sumResult.sortBySumWeight();
sumResult.fillNormalizedIndicesArray();
sumResult.simplifyExpression();

// substitute cumulative derivations on 'N' with "powered" derivations
sumResult.substituteItemAndDerivativesWithData(SUBST.HNABLAN);
sumResult.sortBySumWeight();
sumResult.fillNormalizedIndicesArray();
sumResult.simplifyExpression();

console.log('');
sumResult.showNormalized(OPT.SHOW, 'R(u=parallel) = ');

console.log('');
console.log('The expression contains ' + sumResult.getListLength() + ' terms')

latex = sumResult.getLaTeXEquation(OPT.LTX, 'R_{4,4}(u=\\text{parallel}) = ');
// console.log(latex);
FH.saveToFileWithSureFolder(REL_PATH + FILES.TEX.R4_4_u_parallel, latex);



collectedSum.addTo(sumResult);


FH.showSubsubtitle('Save split files');

collectedSum.sortBySumWeight();
collectedSum.fillNormalizedIndicesArray();
collectedSum.simplifyExpression();

FC.saveSplitSum(collectedSum, PROTO.u, '-u-parallel', REL_PATH);



FH.showSubtitle('N = 0');


FH.showSubsubtitle('Nabla 0');

// nabla0, use the simplified expression with N = zero
sumResult = FC.restoreFromFile(REL_PATH + FILES.JSON.R4_N0[0]);
sumResult.fillNormalizedIndicesArray();

console.log('The expression contains ' + sumResult.getListLength() + ' terms')

// u => parallel
sumResult.substituteItemAndDerivativesWithData(SUBST.uToConstant);
sumResult.sortBySumWeight();
sumResult.fillNormalizedIndicesArray();
sumResult.simplifyExpression();

// substitute cumulative derivations on 'q' with "powered" derivations
sumResult.substituteItemAndDerivativesWithData(SUBST.HNABLAq);
sumResult.sortBySumWeight();
sumResult.fillNormalizedIndicesArray();
sumResult.simplifyExpression();

console.log('');
sumResult.showNormalized(OPT.SHOW, 'R(u=parallel, N=0) = ');

console.log('');
console.log('The expression contains ' + sumResult.getListLength() + ' terms')

latex = sumResult.getLaTeXEquation(OPT.LTX, 'R_{4,0}(u=\\text{parallel}, N=0) = ');
// console.log(latex);
FH.saveToFileWithSureFolder(REL_PATH + FILES.TEX.R4_0_u_parallel_N0, latex);


// Nabla 1 is already 0

FH.showSubsubtitle('Nabla 2');


// nabla2, use the simplified expression with N non zero
sumResult = FC.restoreFromFile(REL_PATH + FILES.JSON.R4_N0[2]);
sumResult.fillNormalizedIndicesArray();

console.log('The expression contains ' + sumResult.getListLength() + ' terms')

// u => parallel
sumResult.substituteItemAndDerivativesWithData(SUBST.uToConstant);
sumResult.sortBySumWeight();
sumResult.fillNormalizedIndicesArray();
sumResult.simplifyExpression();

console.log('');
sumResult.showNormalized(OPT.SHOW, 'R(u=parallel, N=0) = ');

console.log('');
console.log('The expression contains ' + sumResult.getListLength() + ' terms')

latex = sumResult.getLaTeXEquation(OPT.LTX, 'R_{4,2}(u=\\text{parallel}, N=0) = ');
// console.log(latex);
FH.saveToFileWithSureFolder(REL_PATH + FILES.TEX.R4_2_u_parallel_N0, latex);


FH.showSubsubtitle('Nabla 3');

// nabla3, use the simplified expression with N = zero
sumResult = FC.restoreFromFile(REL_PATH + FILES.JSON.R4_N0[3]);
sumResult.fillNormalizedIndicesArray();

console.log('The expression contains ' + sumResult.getListLength() + ' terms')

// u => parallel
sumResult.substituteItemAndDerivativesWithData(SUBST.uToConstant);
sumResult.sortBySumWeight();
sumResult.fillNormalizedIndicesArray();
sumResult.simplifyExpression();

console.log('');
sumResult.showNormalized(OPT.SHOW, 'R(u=parallel, N=0) = ');

console.log('');
console.log('The expression contains ' + sumResult.getListLength() + ' terms')

latex = sumResult.getLaTeXEquation(OPT.LTX, 'R_{4,3}(u=\\text{parallel}, N=0) = ');
// console.log(latex);
FH.saveToFileWithSureFolder(REL_PATH + FILES.TEX.R4_3_u_parallel_N0, latex);



FH.showSubsubtitle('Nabla 4');

// nabla4, use the simplified expression with N non zero
sumResult = FC.restoreFromFile(REL_PATH + FILES.JSON.R4[4]);
sumResult.fillNormalizedIndicesArray();

console.log('The expression contains ' + sumResult.getListLength() + ' terms')

// u => parallel
sumResult.substituteItemAndDerivativesWithData(SUBST.uToConstant);
sumResult.sortBySumWeight();
sumResult.fillNormalizedIndicesArray();
sumResult.simplifyExpression();

console.log('');
sumResult.showNormalized(OPT.SHOW, 'R(u=parallel, N=0) = ');

console.log('');
console.log('The expression contains ' + sumResult.getListLength() + ' terms')

latex = sumResult.getLaTeXEquation(OPT.LTX, 'R_{4,4}(u=\\text{parallel}, N=0) = ');
// console.log(latex);
FH.saveToFileWithSureFolder(REL_PATH + FILES.TEX.R4_4_u_parallel_N0, latex);


let endTime = Date.now();
durationTime = endTime - startTime;
console.log('Duration (ms): ' + durationTime);


FH.showTitle('END: ' + path.basename(__filename));
