/*
Copyright 2019, Thierry Masson, Bruno Iochum

Contact: thierry.masson@cpt.univ-mrs.fr

This file is part of a computer program whose purpose is to help to compute
asymptotic expansion coefficients of the heat-trace for a nonminimal Laplace type operator
using a method developed and exposed in the paper
"Heat coefficient a_4 for nonminimal Laplace type operators" by Bruno Iochum and Thierry Masson.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

let REL_PATH = '../';

let startTime = Date.now();

const fs = require('fs');
const path = require('path');
const chalk = require('chalk');
const {GLOBAL, CONST, DISPLAY_SYMBOLS, OPT, FILES} = require(REL_PATH + 'INIT-FILES/options');
const {FH} = require(REL_PATH + 'FUNCTIONS-FILES/functions-helpers');

const {
				Polynomials,
				RationalExpression
			} = require(REL_PATH + 'MATH-OBJECTS/MathRationalExpression');

const {
				MathElement,
				MathOperator,
				MathElement_one,
			} = require(REL_PATH + 'MATH-OBJECTS/MathElement');

const {
				MathExpression,
				MathProduct,
				MathTensorProduct,
				MathSum,
				MathArgument
			} = require(REL_PATH + 'MATH-OBJECTS/MathExpression');

const {PROTO, SUBST, RULE_DATA, SELECTION} = require(REL_PATH + 'INIT-FILES/initialization-data');

const {FC} = require(REL_PATH + 'FUNCTIONS-FILES/functions-computation');
const {FI} = require(REL_PATH + 'FUNCTIONS-FILES/functions-input');


FH.showTitle('BEGINNING: ' + path.basename(__filename));


partialStartTime = Date.now();


let cutPasteTxt =   '----- cut and paste below -----\n';
let noCutPasteTxt = '------- no cut and paste ------\n';


FH.showSubtitle('EXPORT TO MATHEMATICA');

// length of minimal pattern
let initialLevel = 1;
// length of maximum pattern
let finalLevel = 4;
// shift for maximum level compared to pattern level
let shiftLevel = 4;

let finalMatrixString = '';
let finalBasisSubstitutionString = '';
// loop over patterns
for (let patternLevel=initialLevel; patternLevel<=finalLevel; patternLevel++) {
	console.log('pattern level: ' + patternLevel);
	let initialPattern = [];
	for (let i=0; i<patternLevel; i++) {
		initialPattern.push(CONST.ARRAY_LETTERS_SHORT[i]);
	};
	console.log('initialPattern: ' + JSON.stringify(initialPattern));

	for (let targetLevel = patternLevel+1; targetLevel<=patternLevel + shiftLevel; targetLevel++) {
		console.log('target level: ' + targetLevel);

		let matrix = null;
		let initialBasis = null;
		let finalBasis = null;
		let obj = null;

		obj = FC.getTransitionMatrixAndBasis(initialPattern, targetLevel-1, targetLevel, PROTO.u);

		finalMatrixString += 'M[' + initialPattern.join(',') + ',' + (targetLevel-1) + ',' + targetLevel + '] = '
		finalMatrixString += FH.convertDoubleArrayToMathematicaString(obj.matrix);
		finalMatrixString += ';\n'

		obj = FC.getFullTransitionMatrixAndBasis(initialPattern, targetLevel, PROTO.u);

		finalMatrixString += 'M[' + initialPattern.join(',') + ',' + targetLevel + '] = '
		finalMatrixString += FH.convertDoubleArrayToMathematicaString(obj.matrix);
		finalMatrixString += ';\n'

		let mathematicaString = FH.createMathematicaSubstitutionsForBasis(obj.finalBasis, 'x', 'B', initialPattern);
		finalBasisSubstitutionString += mathematicaString;

	};
	console.log('');

};

// console.log(finalMatrixString);
// console.log(finalBasisSubstitutionString);

textToSave = finalMatrixString + finalBasisSubstitutionString;

FH.saveToFileWithSureFolder(REL_PATH + 'MATHEMATICA/UNCLASSIFIED/mathematica-basis-data.txt', textToSave);


let endTime = Date.now();
durationTime = endTime - startTime;
console.log('Duration (ms): ' + durationTime);


FH.showTitle('END: ' + path.basename(__filename));
