/*
Copyright 2019, Thierry Masson, Bruno Iochum

Contact: thierry.masson@cpt.univ-mrs.fr

This file is part of a computer program whose purpose is to help to compute
asymptotic expansion coefficients of the heat-trace for a nonminimal Laplace type operator
using a method developed and exposed in the paper
"Heat coefficient a_4 for nonminimal Laplace type operators" by Bruno Iochum and Thierry Masson.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

let REL_PATH = '../';

const fs = require('fs');
const path = require('path');
const {GLOBAL, CONST, OPT, FILES} = require(REL_PATH + 'INIT-FILES/options');

// all the functions defined here are properties of the object FH
const FH = {};

FH.printProgressMessage = function(progress){
    process.stdout.clearLine();
    process.stdout.cursorTo(0);
    process.stdout.write(progress);
};

FH.createUniqueID = function(idlength, prefix, listID) {
	let arrChars = CONST.ARRAY_LETTERS_SHORT;
	let uniqid = prefix;
	for (let i = 0; i < idlength; i++) {
			uniqid += arrChars[Math.floor(Math.random() * arrChars.length)];
	}
	if (listID.indexOf(uniqid) === -1) {
		listID.push(uniqid);
		return uniqid;
	} else {
		return FH.createUniqueID(idlength, prefix, listID);
	}
};

FH.nextIndiceID = function(arr) {
	let arrChars = CONST.ARRAY_LETTERS_SHORT;
	let l = arr.length;
	let i = l-1;
	let nextID = '';
	let currentID = '';
	for (let i = 0; i < l; i++) {
			currentID += arrChars[arr[i]];
	};
	// console.log('(nextIndiceID) currentID: ' + currentID);
	arr[i]++;
	let v = arr[i];
	while (v > 25) {
		arr[i] = 0;
		i--;
		arr[i]++;
		v = arr[i];
		if (arr[0] > 25) {
			console.log('Maximum indice ID reached');
		};
	};
	for (let i = 0; i < l; i++) {
			nextID += arrChars[arr[i]];
	};
	// console.log(JSON.stringify(arr));
	// console.log('(nextIndiceID) nextID: ' + nextID);

	return nextID;
};

FH.setCurrentIndiceIDFromString = function(strID, arr) {
	let arrChars = CONST.ARRAY_LETTERS_SHORT;
	// determine the string of current indice id
	let l = arr.length;
	let currentID = '';
	for (let i = 0; i < l; i++) {
			currentID += arrChars[arr[i]];
	};
	// console.log('currentID: ' + currentID);
	// nothing to do if strID <= currentID
	if (currentID < strID) {
		let arrStrID = strID.split('');
		for (let i=arrStrID.length-1; i>=0; i--) {
			let idx = arrChars.indexOf(arrStrID[i]);
			if (idx >= 0) {
				arr[i] = idx;
			} else {
				throw "string out of range";
			};
		}; // for in arrStrID
	}; // if
};

// return an array of indices arrIdx in item
// corresponding to the indicesIDn given in indicesIDnArr
FH.getIdxArrFromIndicesIDnArr = function(indicesIDnArr, item) {
	let indicesArrIdx = item.getIndicesArrIdx();
	let arrIdxArr = [];
	for (let indiceIDn of indicesIDnArr) {
		for (let arrIdx of indicesArrIdx) {
			let testIndiceIDn = item.getTextIndiceAt(arrIdx);
			if (testIndiceIDn === indiceIDn) {
				arrIdxArr.push(arrIdx);
				continue;
			};
		};
	};
	return arrIdxArr;
};




FH.addWeight = function(wdata, obj) {
	obj[wdata.id] = {w: wdata.w, letter: wdata.letter};
	let arr = Object.entries(obj);
	// console.log('');
	// console.log('arr (before): ' + JSON.stringify(arr));
	arr.sort(FH.sortWeightData);
	// console.log('arr (sorted): ' + JSON.stringify(arr));
	let i = 0;
	for (let a of arr) {
		obj[a[0]]['idx'] = i;
		i++
	};
	// console.log('obj: ' + JSON.stringify(obj));
}

FH.sortWeightData = function(wd1, wd2) {
	return wd1[1].w - wd2[1].w;
};

FH.constructStringFromWeight = function(arrWeights, weightData, weightOperatorData) {
	// HYPOTHESIS: integers in the arrays of idx < 52
	// => OK for present applications of the code
	// arrWeights = array of [#ME, #applied MO1, #applied MO2, …]
	let arrChars = CONST.ARRAY_LETTERS_LONG;
	let argN = Object.entries(weightData).length;
	let opN = Object.entries(weightOperatorData).length;
	let str = '';
	for (let argIdx=0; argIdx<argN; argIdx++) {
		if (arrWeights[argIdx] == undefined ) {
			for (let opIdx=opN-1; opIdx>=0; opIdx--) {
				// add 'a' for any MathOperator, reverse order
				str += 'a';
			};
			// add 'a' for MathElement
			str += 'a';
		} else {
			for (let opIdx=opN-1; opIdx>=0; opIdx--) {
				// add letter for any MathOperator, reverse order
				if (arrWeights[argIdx][opIdx+1] == undefined) {
					// this operator does not appear
					str += 'a';
				} else {
					// this operator appears
					str += arrChars[arrWeights[argIdx][opIdx+1]]; // #applied operators
				};
			};
			str += arrChars[arrWeights[argIdx][0]]; // #elements
		};
	};
	return str;
};

FH.constructStringFromSumWeight = function(arrWeights, sumWeightData, sumWeightOperatorData) {
	// HYPOTHESIS: integers in the arrays of idx < 52
	// => OK for present applications of the code
	let arrChars = CONST.ARRAY_LETTERS_LONG;
	let l = Object.entries(sumWeightData).length;
	let str = '';
	for (let idx=0; idx<l; idx++) {
		if (arrWeights[idx] == undefined ) {
			str+= 'aa';
		} else {
			str+= arrChars[arrWeights[idx][1]]; // #applied operators
			str+= arrChars[arrWeights[idx][0]]; // #elements
		};
	};
	return str;
};

FH.constructStringFromProductWeight = function(arrWeights, weightData, weightOperatorData) {
	// HYPOTHESIS: integers in the arrays of idx < 52
	// => OK for present applications of the code
	// arrWeights = array of [#ME, #applied MO1, #applied MO2, …]
	let arrChars = CONST.ARRAY_LETTERS_LONG;
	let argN = Object.entries(weightData).length;
	let opN = Object.entries(weightOperatorData).length;
	let str = '';
	for (let argIdx=0; argIdx<argN; argIdx++) {
		if (arrWeights[argIdx] === undefined ) {
			for (let opIdx=opN-1; opIdx>=0; opIdx--) {
				// add 'a' for any MathOperator, reverse order
				str += 'a';
			};
			// add 'a' for MathElement
			str += '+a';
		} else {
			for (let opIdx=opN-1; opIdx>=0; opIdx--) {
				// add letter for every MathOperator, reverse order
				if (arrWeights[argIdx][opIdx+1] === undefined) {
					// this operator does not appear
					str += 'a';
				} else {
					// this operator appears
					str += arrChars[arrWeights[argIdx][opIdx+1]]; // #applied operators
				};
			};
			str += '+';
			str += arrChars[arrWeights[argIdx][0]]; // #elements
		};
		str += '.';
	};
	return str;
};

FH.getSelectionFromSumWeight = function(arrWeights, weightData, weightOperatorData) {
	let selection = [];
	let argN = Object.entries(weightData).length;
	let argObjs = Object.keys(weightData);
	let opN = Object.entries(weightOperatorData).length;
	let opObjs = Object.keys(weightOperatorData);
	for (let argKey of argObjs) {
		// console.log('argKey: ' + argKey);
		let argObj = weightData[argKey];
		let argIdx = argObj.idx;
		if (arrWeights[argIdx] !== undefined) {
			let obj = {};
			obj.elementID = argKey;
			obj.min = arrWeights[argIdx][0];
			obj.max = arrWeights[argIdx][0];
			obj.operators = [];
			for (let opKey of opObjs) {
				let opObj = weightOperatorData[opKey];
				let opIdx = opObj.idx;
				if (arrWeights[argIdx][opIdx+1] !== undefined) {
					let opObj = {};
					opObj.opID = opKey;
					opObj.min = arrWeights[argIdx][opIdx+1];
					opObj.max = arrWeights[argIdx][opIdx+1];
					obj.operators.push(opObj);
				}
			}; // for opKey
			selection.push(obj);
		}; // arrWeights[argIdx] !== undefined
	}; // for argKey
	return selection;
}


FH.constructRegExpFromSelection = function(selection, weightData, weightOperatorData) {
	// selection: obj with two arrays "main" and "post":
	// {
	// 		elementID: id of element (option),
	// 		element: ref to one element => get its id,
	//		min: minimum number of occurence of elements,
	//		max: maximum number of occurence of elements,
	// 		operators: array of obj
	// 			{	opID: operator id (option),
	// 				op: ref to one operator,
	//				min: minimum number of applied operators,
	//				max: maximum number of applied operators
	//	 		}
	// }
	// '[a-zA-Z]{opN+1}' => null items
	// [a-e] => #0 to #4
	// weightData: {"ad":{"w":1,"letter":"𝟙","idx":0},"hh":{"w":10,"letter":"H","idx":3},"ff":{"w":8,"letter":"L","idx":2},"kn":{"w":6,"letter":"q","idx":1}}

	// console.log('weightData: ' + JSON.stringify(weightData));
	// console.log('');
	// console.log('weightOperatorData: ' + JSON.stringify(weightOperatorData));
	// console.log('');

	// console.log('selection: ' + JSON.stringify(selection));

	let arrChars = CONST.ARRAY_LETTERS_LONG;

	let mainSelection = selection.main;
	let postSelection = selection.post;

	let argN = Object.entries(weightData).length;
	let opN = Object.entries(weightOperatorData).length;

	// RegExpr to "find anything"
	let allRE = '[a-zA-Z]';

	let arrMainRE = [];
	let arrPostRE = [];

	// fill with "find anything"
	for (let argIdx=0; argIdx<argN; argIdx++) {
		arrMainRE[argIdx] = [];
		arrPostRE[argIdx] = [];
		// MathElement
		arrMainRE[argIdx][0] = allRE;
		arrPostRE[argIdx][0] = allRE;
		// MathOperators
		for (let opIdx=opN-1; opIdx>=0; opIdx--) {
			arrMainRE[argIdx][opIdx+1] = allRE;
			arrPostRE[argIdx][opIdx+1] = allRE;
		};
	};

	for (let e of mainSelection) {
		// console.log('e: ' + JSON.stringify(e));
		// get element id either directly of by an instance
		let elementID = null;
		if (e.elementID === undefined) {
			elementID = e.element.getID();
		} else {
			elementID = e.elementID;
		};
		// get idx of ME
		let argIdx = weightData[elementID].idx;
		let min = e.min;
		let max = e.max;
		// console.log('min='+min + ' ' +'max='+max);
		// console.log(arrChars[min] + '-' + arrChars[max]);
		arrMainRE[argIdx][0] = '[' + arrChars[min] + '-' + arrChars[max] + ']';
		let opArr = e.operators;
		for (let o of opArr) {
			// console.log('o: ' + JSON.stringify(o));
			// get operator id directly of by an instance
			let opID = null;
			if (o.opID === undefined) {
				opID = o.op.getID();
			} else {
				opID = o.opID;
			};
			// get idx of ME
			let opIdx = weightOperatorData[opID].idx;
			let min = o.min;
			let max = o.max;
			// console.log('min='+min + ' ' +'max='+max);
			// console.log(arrChars[min] + '-' + arrChars[max]);
			arrMainRE[argIdx][opIdx+1] = '[' + arrChars[min] + '-' + arrChars[max] + ']';
		};
		arrMainRE[argIdx].reverse();
	};
	let arrStrMainRE = [];
	for (let argIdx=0; argIdx<argN; argIdx++) {
		arrStrMainRE[argIdx] = arrMainRE[argIdx].join('');
	};
	// arrStrMainRE.reverse();
	let mainStr = arrStrMainRE.join('');

	for (let e of postSelection) {
		// get element id either directly or by an instance
		let elementID = null;
		if (e.elementID === undefined) {
			elementID = e.element.getID();
		} else {
			elementID = e.elementID;
		};
		// get idx of ME
		let argIdx = weightData[elementID].idx;
		let min = e.min;
		let max = e.max;
		// console.log('min='+min + ' ' +'max='+max);
		// console.log(arrChars[min] + '-' + arrChars[max]);
		arrPostRE[argIdx][0] = '[' + arrChars[min] + '-' + arrChars[max] + ']';
		let opArr = e.operators;
		for (let o of opArr) {
			// console.log('o: ' + JSON.stringify(o));
			// get operator id directly of by an instance
			let opID = null;
			if (o.opID === undefined) {
				opID = o.op.getID();
			} else {
				opID = o.opID;
			};
			// get idx of ME
			let opIdx = weightOperatorData[opID].idx;
			let min = o.min;
			let max = o.max;
			// console.log('min='+min + ' ' +'max='+max);
			// console.log(arrChars[min] + '-' + arrChars[max]);
			arrPostRE[argIdx][opIdx+1] = '[' + arrChars[min] + '-' + arrChars[max] + ']';
		};
			arrPostRE[argIdx].reverse();
	};
	let arrStrPostRE = [];
	for (let argIdx=0; argIdx<argN; argIdx++) {
		arrStrPostRE[argIdx] = arrPostRE[argIdx].join('');
	};
	// arrStrPostRE.reverse();
	let postStr = arrStrPostRE.join('');

	return {mainStr: mainStr, postStr: postStr};
};



FH.sortSumStringWeight = function(item1, item2) {
	if(item1.sumStringWeight < item2.sumStringWeight) return -1;
	if(item1.sumStringWeight > item2.sumStringWeight) return 1;
	return 0;
};

FH.sortProductStringWeight = function(item1, item2) {
	if(item1.productStringWeight < item2.productStringWeight) return -1;
	if(item1.productStringWeight > item2.productStringWeight) return 1;
	return 0;
};

// deep copy an object
// https://www.codementor.io/avijitgupta/deep-copying-in-js-7x6q8vh5d

FH.duplicateInstance = function(orig) {
	// make a duplicate of an instance with same constructor.name
	// WARNING: one has to copy/duplicate every item inside to make a true copy.
	// Exploring ES6 Upgrade (p. 207, A. Rauschmayer)
	let origProto = Object.getPrototypeOf(orig);
	return Object.assign(Object.create(origProto), orig);
};

FH.copyData = function(obj) {
	// copy an object of data, containing only [] and {} (no method)
	// only for serializable properties
	// objects, arrays, strings, booleans and numbers
	return JSON.parse(JSON.stringify(obj));
};

FH.copyString = function(str) {
	// https://stackoverflow.com/questions/31712808/how-to-force-javascript-to-deep-copy-a-string
	return (' ' + str).slice(1);
};

FH.alphaSort = function(a, b){
	if(a < b) return -1;
	if(a > b) return 1;
	return 0;
};

// create an array with only one level from an array of arrays with different levels
FH.flattenArray = function(arr) {
	let newArr = [];
	for (e of arr) {
		if (Array.isArray(e)) {
			newArr = newArr.concat(FH.flattenArray(e));
		} else {
			newArr.push(e);
		};
	};
	return newArr;
};

// swap elements i and j in arr
FH.swapArr = function(arr, i, j) {
	if (0 <= i && 0 <= j && i < arr.length && j < arr.length){
		let e = arr[i];
		arr[i] = arr[j];
		arr[j] = e;
	};
	return arr;
};

// compute the signature of a permutation of integers and sort the array
FH.signaturePermutation = function(arr) {
	if (arr.length === 1) {
		return 1;
	}
	let foundSomething = true;
	let sign = 1;
	while(foundSomething) {
		foundSomething = false;
		for (let i=0; i<arr.length-1; i++) {
			if (arr[i] > arr[i+1]) {
				let a = arr[i];
				arr[i] = arr[i+1];
				arr[i+1] = a;
				sign = -sign;
				foundSomething = true;
				break;
			};
		};
	};
	return sign;
};

// get GCD of array of integers
FH.getGCDArray = function(arr) {
	let gcd = function(a,b) {
		// console.log('(a,b) = ' + a + ' ' + b);
		return b ? gcd(b, a%b) : a;
	};
	if (arr[0] === 1 || arr.length === 1) {
		// console.log('case 1');
		return arr[0];
	} else if (arr.length === 2) {
		// console.log('case 2: ' + JSON.stringify(arr));
		return gcd(arr[0], arr[1]);
	} else {
		let gcd01 = gcd(arr[0], arr[1]);
		// console.log('case 3: ' + gcd01);
		let arrNew = [gcd01].concat(arr.slice(2));
		// console.log('case 3: ' + JSON.stringify(arrNew));
		return FH.getGCDArray(arrNew);
	};
};

// collect all the indice objects + target contraction
// given indiceIDRoot and indiceIDtoIndiceObj object
FH.getSiblingIndiceObjects = function(indiceIDRoot, indiceIDtoIndiceObj) {
	let returnedArray = [];
	let i = 0;
	let currentIndiceID = indiceIDRoot + '-' + i;
	while (indiceIDtoIndiceObj[currentIndiceID] !== undefined) {
		returnedArray.push(indiceIDtoIndiceObj[currentIndiceID]);
		i++;
		currentIndiceID = indiceIDRoot + '-' + i;
	};
	return returnedArray;
};

// return target indiceID associated to source indiceID
// given sIndiceID (source indiceID) and arrCont (contraction array)
// return null if not found
FH.getTargetIndiceIDFromSourceIndiceID = function(sIndiceID, arrCont) {
	let tIndiceID = null;
	for (let cont of arrCont) {
		if (cont[0] === sIndiceID) {tIndiceID = cont[1]};
		if (cont[1] === sIndiceID) {tIndiceID = cont[0]};
	};
	return tIndiceID;
}


// update array of contractions contArr with
// mapping of indiceID in mapIndiceIDArr
// mapping can contain full indices maps ['aaaa-0', 'bbbb-1']
// or root maps ['aaaa', 'bbbb']
FH.updateContractionArray = function(contArr, mapIndiceIDArr) {
	let contStr = JSON.stringify(contArr);
	for (let i=0; i < mapIndiceIDArr.length; i++) {
		let oldIndID = mapIndiceIDArr[i][0];
		let newIndID = mapIndiceIDArr[i][1];
		let re = new RegExp(oldIndID, "g");
		contStr = contStr.replace(re, newIndID);
	};
	let newContArr = JSON.parse(contStr);
	// sort each couples
	for (let e of newContArr) {e.sort(FH.alphaSort);};
	return newContArr;
};

// join two arrays of contractions
FH.joinContractions = function(arr1, arr2) {
	// comparison through strings!
	let arr = [];
	// collect all from arr1
	for (e of arr1) { arr.push(e.sort(FH.alphaSort)) };
	let arrStr = JSON.stringify(arr);
	// collect new from arr2
	for (let e of arr2) {
		if ( !arrStr.includes( JSON.stringify(e.sort(FH.alphaSort)) ) ) {
			arr.push(e.sort(FH.alphaSort));
		};
	};
	return FH.copyData(arr); // to be sure to return new objects
}

// flip two indIDn in contraction array
FH.flipIndicesIDnInContractionArray = function(arrIndIDn, arrCont) {
	let txtCont = JSON.stringify(arrCont);
	// console.log('txtCont (before) = ' + txtCont);
	let indIDn0 = arrIndIDn[0];
	let indIDn1 = arrIndIDn[1];
	// console.log('indIDn0 = '+indIDn0);
	// console.log('indIDn1 = '+indIDn1);
	// this will replace indIDn0 in first step
	// this string should not be a possible indiceIDn
	let placeHolder = '0000-xxxx';
	let reg0 = new RegExp(indIDn0, "g");
	txtCont = txtCont.replace(reg0, placeHolder);
	// console.log('txtCont (mid) = ' + txtCont);
	// replace indIDn1 by indIDn0
	let reg1 = new RegExp(indIDn1, "g");
	txtCont = txtCont.replace(reg1, indIDn0);
	// replace placeHolder by indIDn1
	reg0 = new RegExp(placeHolder, "g");
	txtCont = txtCont.replace(reg0, indIDn1);
	// console.log('txtCont (after) = ' + txtCont);
	let newArrCont = JSON.parse(txtCont);
	for (let e of newArrCont) {
		e.sort(FH.alphaSort);
	}
	return newArrCont;
};


// return a string of contractions for "showConsole"
FH.formatContractionDisplay = function(arr) {
	let str = '';
	if (arr.length > 0) {
		for (let i=0; i < arr.length; i++) {
			str += '❮' + arr[i][0] + '|' + arr[i][1] + '❯';
		};
	};
	return str;
}

// create a pos real number from an array of indices of an element
// and a base number (power of tens)
// integer part in the position of the MathElement
// decimal part represents the level of applied MathOperator
FH.createRealNumberFromIndicesArray = function(arrIdx, base) {
	// arrIdx: array of idx to identify an item in a MathExpression
	// arrange such that
	// pos for postElement < pos for main < pos for preElement
	let r = base; // for main list, between preElement and postElement
	let l = 0;
	let newArrIdx = arrIdx;
	if (arrIdx[0] === -1) {
		// postElement
		r = 0; // force to be first in the sorting
		newArrIdx = arrIdx.slice(1);
	};
	if (arrIdx[0] === -2) {
		// preElement
		// force to be after the main elements
		// at least base**3 since newArrIdx.length smaller
		r = base*base*base;
		newArrIdx = arrIdx.slice(1);
	};
	if (newArrIdx[0] >= 0) {
		for (idx of newArrIdx) {
			if (idx >= 0) {
				r = r*base + idx;
				l++;
			} else {
				r = r + (1 + 0.95/idx);
			};
		};
		return r;
	};
	return 0;
};

// get the reduced permutations used to construct tensor G
FH.reducedPermutations = function(arr) {
	let arrPerm = [];
	if (arr.length === 2) {
		arrPerm.push(arr);
	};
	for (let i=1; i<arr.length; i++) {
		let arrRoot = [arr[0], arr[i]];
		// create copy
		let arrNew = arr.concat([]);
		// remove content of arrRoot
		arrNew.splice(i,1);
		arrNew.splice(0,1);
		let arrTails = FH.reducedPermutations(arrNew);
		for (let a of arrTails) {
			let arrConcat = arrRoot.concat(a);
			arrPerm.push(arrConcat);
		};
	};
	return arrPerm;
};

// factorial function
FH.factorial = function(x) {
	if (x === 0) {
		return 1;
	} else {
		return x*FH.factorial(x-1);
	};
};

FH.toStringPercent = function(r) {
	let p = Math.round(r*100);
	return p + '%';
};

FH.padStringOnRight = function(str, pad, len) {
	if (len + 1 >= str.length) {
		str = str + Array(len+1-str.length).join(pad);
	};
	return str;
};

// convert int to a string with leading zero in order to get digits chars
FH.convertToString = function(int, digits) {
	let string = '' + int;
	while (string.length < digits) {
		string = '0' + string;
	};
	return string;
};

// from an array, create a string
// that can be cut and pasted into Mathematica as a matrix
FH.convertArrayToMathematicaString = function(arr) {
	let mathStr = '{' + arr.join(', ') + '}';
	return mathStr;
};

// from an array of arrays, create a string
// that can be cut and pasted into Mathematica as a matrix
FH.convertDoubleArrayToMathematicaString = function(arr) {
	let mathStrArr = [];
	for (let row of arr) {
		mathStrArr.push('{' + row.join(', ') + '}');
	};
	let mathStr = '{' + mathStrArr.join(', ') + '}';
	return mathStr;
};

// from an array of strings, create a Mathematica expression
// as tensor products
FH.convertArrayToMathematicaTensorProductString = function(arr) {
	let str = arr.join(', ');
	str = CONST.MATH_TENSOR_PRODUCT
					+ CONST.MATH_BRACKET_OPEN
					+ str
					+ CONST.MATH_BRACKET_CLOSE;
	return str;
};

// create an array of length l with 0
FH.createNullArrayWithLength = function(l) {
	let arr = [];
	for (let i=0; i<l; i++) {
		arr.push(0);
	};
	return arr;
};

// create a collection of substitutions to manipulate a basis in Mathematica
// 3 presentation of basis elements and substitutions between them are possible:
// tensor product representation
// letter with indice
// vector with 1 at correct position
// the pattern is used for the Mathematica notation
FH.createMathematicaSubstitutionsForBasis = function(arrBasis, letterRoot, nameRoot, pattern) {
	// console.log('pattern: ' + JSON.stringify(pattern));
	let N = arrBasis.length;
	let collectedBasis = {};
	for (let i=0; i<N; i++) {
		let l = arrBasis[i].length;
		if (collectedBasis[l] === undefined) {
			collectedBasis[l] = [arrBasis[i]];
		} else {
			collectedBasis[l].push(arrBasis[i]);
		};
	};

	let argArr = [];
	for (let c of pattern) {
		// console.log('c: ' + c);
		argArr.push(c);
	};
	let patternStr = argArr.join('');
	// console.log('patternStr: ' + patternStr);

	let mathematicaStringArr = [];
	for (let k in collectedBasis) {
		let BTL = [];
		let BTV = [];
		let BLT = [];
		let BLV = [];
		let BVT = [];
		let BVL = [];
		let currentArgArr = FH.copyData(argArr);
		let currentBasisArr = collectedBasis[k];
		let cN = currentBasisArr.length;
		for (let i=0; i<cN; i++) {
			let letter = letterRoot + currentBasisArr[i].length + patternStr + (i+1);
			// console.log('letter: ' + letter);
			let tensor = FH.convertArrayToMathematicaTensorProductString(currentBasisArr[i]);
			// console.log(tensor);
			let vector = FH.createNullArrayWithLength(N);
			vector[i] = 1;
			vector = FH.convertArrayToMathematicaString(vector);
			// console.log('vector: ' + vector);
			BTL.push(tensor + ' -> ' + letter);
			BTV.push(tensor + ' -> ' + vector);
			BLT.push(letter + ' -> ' + tensor);
			BLV.push(letter + ' -> ' + vector);
			BVT.push(vector + ' -> ' + tensor);
			BVL.push(vector + ' -> ' + letter);
		};
		let mathematicaString = '';
		let l = currentBasisArr[0].length;
		currentArgArr.push(l + '');
		argStr = currentArgArr.join(',');
		mathematicaString += nameRoot + 'TL[' + argStr + '] = '
													+ FH.convertArrayToMathematicaString(BTL) + ';\n';
		mathematicaString += nameRoot + 'TV[' + argStr + '] = '
													+ FH.convertArrayToMathematicaString(BTV) + ';\n';
		mathematicaString += nameRoot + 'LT[' + argStr + '] = '
													+ FH.convertArrayToMathematicaString(BLT) + ';\n';
		mathematicaString += nameRoot + 'LV[' + argStr + '] = '
													+ FH.convertArrayToMathematicaString(BLV) + ';\n';
		mathematicaString += nameRoot + 'VT[' + argStr + '] = '
													+ FH.convertArrayToMathematicaString(BVT) + ';\n';
		mathematicaString += nameRoot + 'VL[' + argStr + '] = '
													+ FH.convertArrayToMathematicaString(BVL) + ';\n';

		mathematicaStringArr.push(mathematicaString);
	};

	let mathematicaReturnedString = mathematicaStringArr.join('');

return mathematicaReturnedString;
};

FH.escapeRegExp = function(str) {
// https://stackoverflow.com/questions/1144783/how-to-replace-all-occurrences-of-a-string-in-javascript
  return str.replace(/[.*+?^${}()|[\]\\]/g, "\\$&"); // $& means the whole matched string
};

// https://stackoverflow.com/questions/1144783/how-to-replace-all-occurrences-of-a-string-in-javascript
FH.replaceAll = function(str, find, replace) {
	return str.replace(new RegExp(find.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&'), 'g'), replace);
};

FH.replaceInStringUsingCorrespondance = function(str, arr) {
	let newStr = str;
	for (let c of arr) {
		// console.log(JSON.stringify(c));
		// let re = new RegExp(c[0], 'g');
		// newStr = newStr.replace(re, c[1]);
		newStr = newStr.split(c[0]).join(c[1]);
	};
	return newStr;
};

// save txt to file filePath ensuring before that the folder structure exists
// create this folder structure if necessary
FH.saveToFileWithSureFolder = function(filePath, txt) {
	// https://stackoverflow.com/questions/13542667/create-directory-when-writing-to-file-in-node-js
	function ensureDirectoryExistence(filePath) {
		var dirname = path.dirname(filePath);
		if (fs.existsSync(dirname)) {
			return true;
		}
		ensureDirectoryExistence(dirname);
		fs.mkdirSync(dirname);
	};

	ensureDirectoryExistence(filePath);
	fs.writeFileSync(filePath, txt, 'utf8');
	return;
}


// save item to the given file
// item.toJSON() is fired to get a string representation
FH.saveItemToFile = function(item, file) {
	let txt = item.toJSON();
	FH.saveToFileWithSureFolder(file, txt)
};

// load data from file and parse
FH.getDataFromFile = function(file) {
	let data = fs.readFileSync(file, 'utf8');
	// console.log(data);
	return data;
};


// titles and subtitles
FH.showTitle = function(text) {
	let width = OPT.TITLE.widthTitle;
	let textLength = text.length;
	let left = Math.floor(0.5*(width - 2 - textLength));
	let right = width - 2 - textLength - left;
	if (right<0) {right = 0;};
	console.log('');
	console.log('-'.repeat(width));
	let toDisplay = '-'.repeat(left) + ' ' + text + ' ' + '-'.repeat(right);
	console.log(toDisplay);
	console.log('-'.repeat(width));
	console.log('');
};

FH.showSubtitle = function(text) {
	let width = OPT.TITLE.widthSubtitle;
	let textLength = text.length;
	let left = Math.floor(0.5*(width - 2 - textLength));
	let right = width - 2 - textLength - left;
	if (right<0) {right = 0;};
	console.log('');
	console.log('-'.repeat(width));
	let toDisplay = '-'.repeat(left) + ' ' + text + ' ' + '-'.repeat(right);
	console.log(toDisplay);
	console.log('');
};

FH.showSubsubtitle = function(text) {
	let width = OPT.TITLE.widthSubsubtitle;
	let textLength = text.length;
	let left = Math.floor(0.5*(width - 2 - textLength));
	let right = width - 2 - textLength - left;
	if (right<0) {right = 0;};
	console.log('');
	let toDisplay = '-'.repeat(left) + ' ' + text + ' ' + '-'.repeat(right);
	console.log(toDisplay);
	console.log('');
};

FH.showZone= function() {
	console.log(chalk.red.bold("*".repeat(OPT.TITLE.widthZone)));
};

module.exports = {
	FH: FH,
};
