/*
Copyright 2019, Thierry Masson, Bruno Iochum

Contact: thierry.masson@cpt.univ-mrs.fr

This file is part of a computer program whose purpose is to help to compute
asymptotic expansion coefficients of the heat-trace for a nonminimal Laplace type operator
using a method developed and exposed in the paper
"Heat coefficient a_4 for nonminimal Laplace type operators" by Bruno Iochum and Thierry Masson.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

let REL_PATH = '../';

const {GLOBAL, CONST, DISPLAY_SYMBOLS, OPT, FILES} = require(REL_PATH + 'INIT-FILES/options');
const {PROTO} = require(REL_PATH + 'INIT-FILES/initialization-data');

const {
				Polynomials,
				RationalExpression
			} = require(REL_PATH + 'MATH-OBJECTS/MathRationalExpression');

const {
				MathElement,
				MathOperator
			} = require(REL_PATH + 'MATH-OBJECTS/MathElement');

const {
				MathExpression,
				MathProduct,
				MathTensorProduct,
				MathSum,
				MathArgument,
			} = require(REL_PATH + 'MATH-OBJECTS/MathExpression');


// contains placeholder tensors and operators
// needed not be included in real computations
const FS_DATA = {};


let tensorUp = new MathElement();
tensorUp
	.setID('Tu')
	.setLetter('T')
	.setMathLetter('T')
	.setLaTeXLetter('T')
	.setProductKindToNonCommutative()
	.addIndiceUp()
	.setSumWeight(60)
	.setProductWeight(60);

let tensorDown = new MathElement();
tensorDown
	.setID('Td')
	.setLetter('T')
	.setMathLetter('T')
	.setLaTeXLetter('T')
	.setProductKindToNonCommutative()
	.addIndiceDown()
	.setSumWeight(61)
	.setProductWeight(61);

let Christoffel = new MathElement();
Christoffel
	.setID('Gamma')
	.setLetter('𝚪')
	.setMathLetter('Gamma')
	.setLaTeXLetter('\\Gamma')
	.setProductKindToCommutative()
	.addIndiceUp()
	.addIndiceDown()
	.addIndiceDown()
	.setSumWeight(35)
	.setProductWeight(35);

FS_DATA.placeHolderTensorData = {
	tensorUp: tensorUp,
	tensorDown:tensorDown
};

FS_DATA.operatorData = {
	AppliedOperator: PROTO.nabla,
	NonCommutative: PROTO.nabla,
	Commutative: PROTO.partial,
	Christoffel: Christoffel
};

FS_DATA.operatorDataNCT = {
	AppliedOperator: PROTO.nabla,
	NonCommutative: PROTO.delta,
	Commutative: PROTO.partial,
	Christoffel: Christoffel
};




module.exports = {
	FS_DATA: FS_DATA,
};
