#/usr/bin/sh !

# Copyright 2019, Thierry Masson, Bruno Iochum
#
# Contact: thierry.masson@cpt.univ-mrs.fr
#
# This file is part of a computer program whose purpose is to help to compute
# asymptotic expansion coefficients of the heat-trace for a nonminimal Laplace type operator
# using a method developed and exposed in the paper
# "Heat coefficient a_4 for nonminimal Laplace type operators" by Bruno Iochum and Thierry Masson.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.




cd 'COMPUTATIONS-0-SUBSTITUTIONS/'

# compute substitutions used in computations
echo 'substitutions Gamma'
node compute-substitutions-1-Gamma.js

echo 'substitutions H'
node compute-substitutions-2-H.js

echo 'substitutions L'
node compute-substitutions-3-L.js

echo 'substitutions nabla -> hnabla u'
node compute-substitutions-4-hnabla-u.js

echo 'substitutions nabla -> hnabla N'
node compute-substitutions-5-hnabla-N.js

echo 'substitutions nabla -> hnabla q'
node compute-substitutions-6-hnabla-q.js

# export substitutions to latex
echo 'substitutions by hand'
node show-substitutions.js


cd '../COMPUTATIONS-1-MAIN-R2-R4/'

echo 'R2'
node compute-R2.js

# this one can take a long time…
echo 'R4 step 0 => save'
node compute-R4-step-0-save.js


echo 'R4 nabla 4 in post element'
node compute-R4-4.js


echo 'R4 nabla 3 in post element'
node compute-R4-3.js


echo 'R4 nabla 2 in post element'
node compute-R4-2.js

echo 'R4 nabla 2,3,4 in post element, split'
node compute-R4-234-split.js

echo 'R4 nabla 1 in post element (should return 0)'
node compute-R4-1.js


echo 'R4 nabla 0: step 1 => non simplified expressions'
node compute-R4-0-step-1-save-ns.js

echo 'R4 nabla 0: step 2 => simplified expressions'
node compute-R4-0-step-2-simplify.js

echo 'R4 nabla 0: step 3 => latex'
node compute-R4-0-step-3-latex.js

echo 'R4 nabla 0: step 4 => split'
node compute-R4-0-step-4-split.js


echo 'R4: double derivations on u (symmetric/antisymmetric parts)'
node compute-R4-double-derivations-of-u.js



cd '../COMPUTATIONS-2-u-PARALLEL-R4/'

# special case u parallel (all derivations of u go to 0)
echo 'R4 for u parallel'
node compute-R4-u-parallel.js


cd '../COMPUTATIONS-3-NCT-R2-R4/'

echo 'NCT: substitutions'
node compute-substitutions-NCT.js

echo 'NCT: R2'
node compute-R2-NCT.js


# check all the results against proposed simplifications
cd '../COMPUTATIONS-1-MAIN-R2-R4/'
echo 'R4: check simplified expressions'
node check-simplifications-R4.js

cd '../COMPUTATIONS-2-u-PARALLEL-R4/'
echo 'R4 for u parallel: check simplified expressions'
node check-simplifications-R4-u-parallel.js



cd '../'

echo 'the end'
